@@ -10,29 +10,38 @@
 #define __XML_XSLT_NAMESPACES_H__
 
 #include <libxml/tree.h>
+#include "xsltexports.h"
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-void		xsltNamespaceAlias	(xsltStylesheetPtr style,
+XSLTPUBFUN void XSLTCALL
+		xsltNamespaceAlias	(xsltStylesheetPtr style,
 					 xmlNodePtr node);
-xmlNsPtr	xsltGetNamespace	(xsltTransformContextPtr ctxt,
+XSLTPUBFUN xmlNsPtr XSLTCALL	
+		xsltGetNamespace	(xsltTransformContextPtr ctxt,
 					 xmlNodePtr cur,
 					 xmlNsPtr ns,
 					 xmlNodePtr out);
-xmlNsPtr	xsltGetSpecialNamespace	(xsltTransformContextPtr ctxt,
+XSLTPUBFUN xmlNsPtr XSLTCALL	
+		xsltGetSpecialNamespace	(xsltTransformContextPtr ctxt,
 					 xmlNodePtr cur,
 					 const xmlChar *URI,
 					 const xmlChar *prefix,
 					 xmlNodePtr out);
-xmlNsPtr	xsltCopyNamespace	(xsltTransformContextPtr ctxt,
+XSLTPUBFUN xmlNsPtr XSLTCALL	
+		xsltCopyNamespace	(xsltTransformContextPtr ctxt,
 					 xmlNodePtr node,
 					 xmlNsPtr cur);
-xmlNsPtr	xsltCopyNamespaceList	(xsltTransformContextPtr ctxt,
+XSLTPUBFUN xmlNsPtr XSLTCALL	
+		xsltCopyNamespaceList	(xsltTransformContextPtr ctxt,
 					 xmlNodePtr node,
 					 xmlNsPtr cur);
-void		xsltFreeNamespaceAliasHashes(xsltStylesheetPtr style);
+XSLTPUBFUN void XSLTCALL		
+		xsltFreeNamespaceAliasHashes
+					(xsltStylesheetPtr style);
+
 #ifdef __cplusplus
 }
 #endif
