@@ -1165,14 +1165,13 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 	}
 	if (cur->type == XML_ELEMENT_NODE) {
 	    int exclPrefixes;
-	    xmlChar *prefix;
 
 	    exclPrefixes = xsltParseStylesheetExcludePrefix(style, cur);
 	    if (IS_XSLT_ELEM(cur)) {
 		xsltStylePreCompute(style, cur);
 		if (IS_XSLT_NAME(cur, "text")) {
 		    for (;exclPrefixes > 0;exclPrefixes--)
-			prefix = exclPrefixPop(style);
+			exclPrefixPop(style);
 		    goto skip_children;
 		}
 	    }
@@ -1222,7 +1221,7 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 	    if (exclPrefixes > 0) {
 		xsltPrecomputeStylesheet(style, cur->children);
 		for (;exclPrefixes > 0;exclPrefixes--)
-		    prefix = exclPrefixPop(style);
+		    exclPrefixPop(style);
 		goto skip_children;
 	    }
 	} else if (cur->type == XML_TEXT_NODE) {
@@ -1315,7 +1314,7 @@ xsltGatherNamespaces(xsltStylesheetPtr style) {
 		    URI = xmlHashLookup(style->nsHash, ns->prefix);
 		    if ((URI != NULL) && (!xmlStrEqual(URI, ns->href))) {
 			xsltTransformError(NULL, style, cur,
-	     "Namespaces prefix %s used for multiple namespaces\n");
+	     "Namespaces prefix %s used for multiple namespaces\n",ns->prefix);
 			if (style != NULL) style->warnings++;
 		    } else if (URI == NULL) {
 			xmlHashUpdateEntry(style->nsHash, ns->prefix,
@@ -1516,16 +1515,14 @@ skip_children:
 	if ((IS_XSLT_ELEM(cur)) && (IS_XSLT_NAME(cur, "param"))) {
 	    xmlNodePtr param = cur;
 
-            cur = cur->next;
 	    xsltTransformError(NULL, style, cur,
 		"xsltParseTemplateContent: ignoring misplaced param element\n");
 	    if (style != NULL) style->warnings++;
+            cur = cur->next;
 	    xmlUnlinkNode(param);
 	    xmlFreeNode(param);
-	    continue;
 	} else
 	    break;
-	cur = cur->next;
     }
 }
 
